# robobin/connection_manager.py
import socket
import threading
import time

class ConnectionManager:
    def __init__(self, api_node, udp_ip="255.255.255.255", udp_port=5005, listen_port=5006):
        self.api_node = api_node
        self.UDP_IP = udp_ip
        self.UDP_PORT = udp_port
        self.LISTEN_PORT = listen_port
        self.stop_event = threading.Event()

    def start(self):
        """Starts listening for connections and broadcasting presence."""
        self.listen_thread = threading.Thread(target=self.listen_for_connections)
        self.broadcast_thread = threading.Thread(target=self.broadcast_presence)
        self.listen_thread.start()
        self.broadcast_thread.start()

    def listen_for_connections(self):
        """Listens for UDP 'CONNECT' messages and sets up TCP connections."""
        udp_sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        udp_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        udp_sock.bind(('', self.LISTEN_PORT))

        while not self.stop_event.is_set():
            try:
                data, addr = udp_sock.recvfrom(1024)
                if data.decode() == "CONNECT":
                    print(f"Connection request from {addr}")
                    tcp_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    tcp_sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                    tcp_sock.bind(('', self.LISTEN_PORT))
                    tcp_sock.listen(1)
                    client_socket, client_addr = tcp_sock.accept()
                    print(f"Client connected from {client_addr}")
                    threading.Thread(target=self.api_node.handle_client_connection, args=(client_socket,)).start()
            except socket.timeout:
                continue
            except OSError:
                break

        udp_sock.close()
        print("Stopped listening for connections.")

    def broadcast_presence(self):
        """Broadcasts presence periodically over UDP."""
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

        while not self.stop_event.is_set():
            try:
                location = "(0,0)" #At some point this will be retrieved from navigation node
                message = f"ROBOBIN_PRESENT {location}".encode()
                sock.sendto(message, (self.UDP_IP, self.UDP_PORT))
                print("Broadcasting presence.")
                time.sleep(5)
            except OSError:
                break

        sock.close()
        print("Stopped broadcasting presence.")

    def stop(self):
        """Stops the connection manager."""
        #@TODO: Implement stoppage gracefully. Right now 
        self.stop_event.set()
    
if __name__ == "__main__":
    ConnectionManager(None).start()
